import { system, world, EntityComponentTypes, GameMode } from "@minecraft/server";

function getLinkedEntity(mainEntity) {
    let linkedEntity = null;
    const dimension = mainEntity.dimension;
    dimension.getEntities({
        location: mainEntity.location,
        closest: 2
    }).forEach((entity) => {
        if (entity.hasTag("spectating") && entity.typeId !== mainEntity.typeId)
            linkedEntity = entity;
    });
    return linkedEntity;
}

function dismount(mainEntity) {
    mainEntity?.removeTag("spectating");
    if (mainEntity?.typeId !== "minecraft:player")
        return;
    mainEntity?.setGameMode(GameMode.spectator);
    mainEntity?.addEffect("instant_health", 1, {amplifier: 254, showParticles: false});
}

world.afterEvents.entityHurt.subscribe(({ damage, damageSource, hurtEntity }) => {
    if (!hurtEntity.hasTag("spectating"))
        return;
    const linkedEntity = getLinkedEntity(hurtEntity);

    if (linkedEntity?.typeId === "minecraft:player") {
        linkedEntity?.addEffect("resistance", 1, {amplifier: 254, showParticles: false});
        linkedEntity?.applyDamage(1);
        linkedEntity?.addEffect("resistance", 1, {amplifier: 254, showParticles: false});
    } else {
        linkedEntity?.applyDamage(damage);
    }
});

world.afterEvents.itemUse.subscribe(({ itemStack, source }) => {
    if (!source.hasTag("spectating"))
        return;
    dismount(source);
    source.addTag("debounce");
    const linkedEntity = getLinkedEntity(source);
    linkedEntity?.removeTag("spectating");
});

system.runInterval(() => {
    world.getPlayers().forEach((player) => {
        if (player.hasTag("spectating")) {
            const linkedEntity = getLinkedEntity(player);
            if (linkedEntity !== null) {
                player.runCommand(`title @s actionbar Spectating ${linkedEntity.nameTag != '' ? linkedEntity.nameTag : linkedEntity.typeId}...`);
                if (linkedEntity?.typeId !== "minecraft:player") {
                    const linkedEntityHealth = linkedEntity?.getComponent(EntityComponentTypes.Health);
                    const playerHealth = player.getComponent(EntityComponentTypes.Health);
                    if (linkedEntityHealth?.currentValue > playerHealth?.effectiveMax) {
                        const mobHealthAmplifier = Math.round((linkedEntityHealth?.effectiveMax-playerHealth?.effectiveMax)/4) -1;
                        player.addEffect("health_boost", 40, {amplifier: mobHealthAmplifier, showParticles: false});
                    }
                    if (playerHealth?.currentValue > (playerHealth?.effectiveMin +1)) {
                        playerHealth?.setCurrentValue(linkedEntityHealth?.currentValue);
                    } else {
                        player.addEffect("resistance", 1, {amplifier: 254, showParticles: false});
                    }
                }
            } else {
                dismount(player);
            }
        }

        if (player.hasTag("debounce")) {
            const dimension = player.dimension;
            const closestEntities = dimension.getEntities({
                location: player.location,
                closest: 2,
                maxDistance: 3
            });
            if (closestEntities.length <= 1) {
                player.removeTag("debounce");
            }
        }
    });
}, 2);